# frozen_string_literal: true

module Hermes
  # Nud class
  class Nud
    # Linked to the method of housing_request :
    #   - nud_creations
    include Hermes::Concerns::CommonFieldsConcern
    include Hermes::Concerns::ActivableConcern
    include Hermes::Concerns::SopraXmlSender
    include Hermes::Concerns::SopraXmlSender

    field :applicant_id, type: BSON::ObjectId
    field :department, type: String
    field :creation_date, type: DateTime
    field :department, type: String
    field :unique_number, type: String


    belongs_to(
        :housing_request,
        class_name: 'Hermes::HousingRequest',
        index: true
    )
    validates_presence_of :housing_request

    writer_roles 'Superadmin', 'Administrator'
    reader_roles 'Manager'
    owner_field :user_id

    common_indexes *%i[
      department
      creation_date
      unique_number
      is_active
    ]

    before_save     :before_save_actions
    before_destroy  :before_destroy_actions
    #after_save      :after_save_actions
    after_create    :after_create_actions
    after_destroy   :after_destroy_actions

    # ========================== #
    # - SIDE EFFETCTS HANDLERS - #
    # ========================== #
    def before_save_actions
      update_applicant_id if housing_request_id_changed?
    end

    def before_destroy_actions
      housing_request.nud_ids.delete(id)
      housing_request.save
    end

    def after_create_actions
      housing_request.save
      update_affiliated_scores if oldest_for_department?
      if Rails.env == 'test'
        {exectuted: true, xml: to_xml}
      else
      Hermes::BoClient::FindNudTask.new(nud_id: id)
                                   .execute
        end
    end

    def after_update_actions
      housing_request.save unless applicant_id_changed? # to avoid infinit loop
      update_affiliated_scores if must_update_affiliated_scores?
    end

    def after_destroy_actions
      update_affiliated_scores if oldest_for_department?
    end

    # ============================== #
    # - SIDE EFFETCTS DEPENDENCIES - #
    # ============================== #
    def update_applicant_id
      self.applicant_id = housing_request.main_applicant_id
    end

    def oldest_for_department?
      # has the oldest @creation_date
      # WITHIN @demartement.nuds
      # WHERE_SAME @housing_request
      oldest_other = housing_request.nuds
                                    .where(
                                      id: { :$ne => id },
                                      department: department,
                                      creation_date: { :$lte => creation_date }
                                    ).order_by(creation_date: 1)
                                    &.first
                                    &.creation_date
      oldest_other.blank? || oldest_other > creation_date
    end

    # Scores update trigger
    def must_update_affiliated_scores?
      # IF
      #   any of fields list has changed
      # AND
      %w[
        creation_date
        department
        applicant_id
        is_active
      ].any? { |field_name| send("#{field_name}_changed?") } &&
        oldest_for_department?
    end

    # Affiliated scores are all that match
    #   SAME @department && @applicant_id
    def update_affiliated_scores
      ToucheScoresJob.perform_later(
        Hermes::Score.only(:id, :department, :applicant_id)
                     .where(
                       department: department,
                       applicant_id: housing_request.main_applicant_id
                     ).pluck(:id)
      )
    end


   
  end
end
